﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using BeyondBasicUnitTesting.Business;
using System.IO;

namespace BeyondBasicUnitTesting.UnitTests
{
    /// <summary>
    /// Summary description for PersonStubFixture
    /// </summary>
    [TestClass]
    public class RhinoMockDemoFixture : UnitTestBase
    {
        [TestMethod]
        public void RhinoMockDemo_NothinButMocksBaby()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.Stub<IPerson>();
            IPersonFacade facade = repository.Stub<IPersonFacade>();

            facade.IsPersonValid(person);
        }

        [TestMethod]
        public void RhinoMockDemo_MockPersonRealFacade_IsValid()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.Stub<IPerson>();
            IPersonFacade facade = new PersonFacade();

            Assert.IsFalse(facade.IsPersonValid(null), "Null person should not be valid.");
            Assert.IsFalse(facade.IsPersonValid(person), "Person with all null fields should not be valid.");

            CallIsValidAndAssert(person, facade, "Ben", "Day", "benday@benday.com", "ACTIVE", true);
            CallIsValidAndAssert(person, facade, null, "Day", "benday@benday.com", "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", null, "benday@benday.com", "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", "Day", null, "ACTIVE", false);
            CallIsValidAndAssert(person, facade, "Ben", "Day", "benday@benday.com", null, false);
        }

        private void CallIsValidAndAssert(IPerson person, 
            IPersonFacade facade, 
            string firstName, 
            string lastName, 
            string email, 
            string status, 
            bool expectedIsValidResult)
        {
            PopulatePerson(person, firstName, lastName, email, status);

            if (expectedIsValidResult == true)
            {
                Assert.IsTrue(facade.IsPersonValid(person), "Person should be valid.");
            }
            else
            {
                Assert.IsFalse(facade.IsPersonValid(person), "Person should not be valid.");
            }
        }

        private static void PopulatePerson(IPerson person, string firstName, string lastName, string email, string status)
        {
            person.FirstName = firstName;
            person.LastName = lastName;
            person.EmailAddress = email;
            person.Status = status;
        }

        [TestMethod]
        public void RhinoMockDemo_SavePerson()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.Stub<IPerson>();
            IPersonFacade facade = new PersonFacade();

            PopulatePerson(person, "Ben", "Day", "benday@benday.com", "ACTIVE");

            Assert.AreEqual<int>(0, person.Id, "Id should be zero before save.");

            facade.SavePerson(person);

            Assert.AreNotEqual<int>(0, person.Id, "Person id was still zero after save.");
        }

        [TestMethod]
        [ExpectedException(typeof(FileNotFoundException))]
        public void RhinoMockDemo_SavePersonLogsException()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.DynamicMock<IPerson>();

            SetupResult.For(person.FirstName).Throw(new FileNotFoundException());

            repository.ReplayAll();
            
            IPersonFacade facade = new PersonFacade();

            facade.SavePerson(person);
        }

        [TestMethod]
        public void RhinoMockDemo_SavePersonLogsExceptionWithVerification()
        {
            MockRepository repository = new MockRepository();

            IPerson person = repository.DynamicMock<IPerson>();
            ILogger logger = repository.DynamicMock<ILogger>();

            SetupResult.For(person.FirstName).Throw(new FileNotFoundException());
            Expect.Call(delegate{logger.WriteToLog(string.Empty);}).IgnoreArguments();

            repository.ReplayAll();

            IPersonFacade facade = new PersonFacade2(logger);

            try
            {
                facade.SavePerson(person);
            }
            catch (FileNotFoundException)
            {
                // got the exception
            }

            repository.Verify(logger);
        }
    }
}
