using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using BeyondBasicUnitTesting.Business;

namespace BeyondBasicUnitTesting.UnitTests
{
    public class RepositoryMock<T> : IRepository<T> where T : IInt32Id
    {
        private int m_currentIdValue = 0;

        public RepositoryMock()
        {
            Items = new List<T>();
        }

        protected List<T> Items
        {
            get;
            set;
        }

        protected virtual T BeforeReturn(T item)
        {
            return item;
        }

        protected virtual IList<T> BeforeReturn(IList<T> items)
        {
            if (items == null)
            {
                return null;
            }
            else
            {
                foreach (T item in items)
                {
                    BeforeReturn(item);
                }

                return items;
            }
        }

        public IList<T> GetAll()
        {
            return BeforeReturn(Items);
        }

        public T GetById(int id)
        {
            return BeforeReturn((from b in Items
                                 where b.Id == id
                                 select b).FirstOrDefault());
        }

        public void Save(T saveThis)
        {
            if (saveThis == null)
            {
                throw new ArgumentNullException("saveThis", "Argument cannot be null.");
            }

            if (saveThis.Id == 0)
            {
                saveThis.Id = GetNextIdValue();
            }

            if (Items.Contains(saveThis) == false)
            {
                Items.Add(saveThis);
            }
        }

        protected int GetNextIdValue()
        {
            return ++m_currentIdValue;
        }
    }
}
